/*  © 2017 Playful Invention Company */

/////////////////////////
//
// Hidden data reading
//
/////////////////////////

Img = function() {};

Img.drawImageToFit = function(ctx, img){
	var s = ctx.canvas.width/img.naturalWidth;
	ctx.save();
	ctx.scale(s,s);
	ctx.drawImage(img, 0, 0);
	ctx.restore();	
}

Img.getImageData = function(ctx){
	var pixels = ctx.getImageData(0, 0, 500, 300).data;
	var x=0, y=0;
	var signature = readln(20);
	if(signature!='TurtleArt') return 'bad sig';
	return readln(100000);


	function readln(max){
		var res = '';
		var c, cksum=0;
		for (var i=0;i<max;i++) {
			c=rb();
			cksum += c;
			cksum &= 0xffff;
			if (c == 0xff) {
				if  (cksum != rb() + (rb() << 8)) res = "bad checksum";
				return decodeURIComponent(escape(res));
			}
			res = res + String.fromCharCode(c);
		}
		return 'bad readln';
	}

	function rb(){
		var res = 0;
		var idx = (x*4)+y*2000;
		res += (pixels[idx+2]&1);
		res += (pixels[idx+1]&1)<<1;
		res += (pixels[idx]&1)<<2;
		idx = (x*4)+(y+1)*2000;
		res += (pixels[idx+2]&1)<<3;
		res += (pixels[idx+1]&1)<<4;
		res += (pixels[idx]&1)<<5;
		idx = (x*4)+(y+2)*2000;
		res += (pixels[idx+2]&1)<<6;
		res += (pixels[idx+1]&1)<<7;
		x++;
		if (x==500) {x = 0; y+=3;}
		return res
	}

}

Img.setImageData = function(ctx, str){
	var imd = ctx.getImageData(0, 0, 500, 300);
	var pixels = imd.data;
	var x=0, y=0;
	println('TurtleArt');
	println(str);
	ctx.putImageData(imd,0,0);


	function println(str){
		var arr = explode(str);
		arr.push(0xff);
		var cksum = 0;
		for(var i in arr){
			wb(arr[i]);
			cksum+=arr[i];
		}
		wb(cksum&0xff);
		wb((cksum>>8)&0xff);
	}

	function wb(n){
		var idx = (x*4)+y*2000;
		pixels[idx+2]&=0xfe; pixels[idx+2]+=getbit(n,0);
		pixels[idx+1]&=0xfe; pixels[idx+1]+=getbit(n,1);
		pixels[idx]&=0xfe; pixels[idx]+=getbit(n,2);
		pixels[idx+3]=0xff;
		idx = (x*4)+(y+1)*2000;
		pixels[idx+2]&=0xfe; pixels[idx+2]+=getbit(n,3);
		pixels[idx+1]&=0xfe; pixels[idx+1]+=getbit(n,4);
		pixels[idx]&=0xfe; pixels[idx]+=getbit(n,5);
		pixels[idx+3]=0xff;
		idx = (x*4)+(y+2)*2000;
		pixels[idx+2]&=0xfe; pixels[idx+2]+=getbit(n,6);
		pixels[idx+1]&=0xfe; pixels[idx+1]+=getbit(n,7);
		pixels[idx+3]=0xff;
		x++;
		if (x==500) {x = 0; y+=3;}
	}

	function explode(str){
		var res = new Array();
		var ustr = unescape(encodeURIComponent(str));
		for(var i=0;i<ustr.length;i++) res.push(ustr.charCodeAt(i));
		return res;
	}

	function getbit(word, n) { return 1 & (word >> n); }

}


/////////////////////////
//
// Tokenizer
//
/////////////////////////

Tokenizer = function() {};

Tokenizer.tokenize = function(s){
	var tokstr = s;
	var tokoffset=0;
	skipSpace();
	return readList();

	function readList(){
		var a = new Array();
		while(true){
			if(eof()) break;
			var token = readToken();
			if(token==null) break;
			a.push(token);
		}
		return a;
	}

	function readToken(){
		var s = next();
		var n = Number(s);
		if(!isNaN(n)) return n;
		var first = s.charAt(0);
		if(first=="]") return null;
		if(first=="[") return readList();
		return s;
	}

	function next(){
		if(peekChar()=='|') return readVbarString();
		var res='';
		if(delim()) res=nextChar();
		else {
			while(true){
				if(eof()) break;
				if(delim()) break;
				else res+=nextChar();
		}}
		skipSpace();
		return res;
	}

	function readVbarString(){
		nextChar();
		var res='';
		while (true){
			var c=nextChar();
			if(eof()) return res;
			if(c=="|") {skipSpace(); return res;}
			res+=c;
		}
		return null;
	}

	function skipSpace(){
		while(true){
			if(eof()) return;
			var c = peekChar();
			if(" \t\r\n,".indexOf(c)==-1) return;
			nextChar();
		}	
	}

	function delim(){return "()[] \t\r\n".indexOf(peekChar())!=-1;}
	function peekChar(){return tokstr.charAt(tokoffset);}
	function nextChar(){return tokstr.charAt(tokoffset++);}
	function eof() {return tokstr.length==tokoffset;}	
}

/////////////////////////
//
// Splitter
//
/////////////////////////

Splitter = function() {};

Splitter.split = function(str){
	var splstr = str;
	var sploffset=0;
	var a = new Array();
	while(true){
		if(eof()) break;
		if(peekChar()=='[') a.push(readList());
		else if(peekChar()=='|') a.push(readWord());
		skipSpace();
	}
	return a;


	function readList(){
		var res = nextChar();
		var level=1;
		while (true){
			var c=nextChar();
			res+=c;
			if((c==']')&&(--level==0)) return res;
			if(c=='[') level++;
		}
	}

	function readWord(){
		var res = nextChar();
		while (true){
			var c=nextChar();
			res+=c;
			if(c=='|') return res;
		}
	}

	function skipSpace(){
		while(true){
			if(eof()) return;
			var c = peekChar();
			if(" \t\r\n,".indexOf(c)==-1) return;
			nextChar();
		}	
	}

	function delim(){return "()[] \t\r\n".indexOf(peekChar())!=-1;}
	function peekChar(){return splstr.charAt(sploffset);}
	function nextChar(){return splstr.charAt(sploffset++);}
	function eof() {return splstr.length==sploffset;}	

}